//Microchip Solutions v2011-12-05
/******************************************************************************
            USB Custom Demo, Host

This file provides the main entry point to the Microchip USB Custom
Host demo.  This demo shows how a PIC24F system could be used to
act as the host, controlling a USB device running the Microchip Custom
Device demo.

******************************************************************************/

/******************************************************************************
* Filename:        main.c
* Dependancies:    USB Host Driver with Generic Client Driver
* Processor:       PIC24F256GB1xx
* Hardware:        Explorer 16 with USB PICtail Plus
* Compiler:        C30 v2.01/C32 v0.00.18
* Company:         Microchip Technology, Inc.

Software License Agreement

The software supplied herewith by Microchip Technology Incorporated
(the CompanyE for its PICmicro Microcontroller is intended and
supplied to you, the Companys customer, for use solely and
exclusively on Microchip PICmicro Microcontroller products. The
software is owned by the Company and/or its supplier, and is
protected under applicable copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to
civil liability for the breach of the terms and conditions of this
license.

THIS SOFTWARE IS PROVIDED IN AN AS ISECONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


*******************************************************************************/

#include <stdlib.h>
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "usb_config.h"
#include "USB/usb.h"
#include "USB/usb_host_generic.h"
#include "user.h"
#include "timer.h"
#include "DEE Emulation 16-bit.h"

#define DEBUG_MODE

// *****************************************************************************
// *****************************************************************************
// Configuration Bits
// *****************************************************************************
// *****************************************************************************

        _CONFIG1(WDTPS_PS1 & FWPSA_PR32 & WINDIS_OFF & FWDTEN_OFF & ICS_PGx1 & GWRP_OFF & GCP_OFF & JTAGEN_OFF)
        _CONFIG2(POSCMOD_HS & I2C1SEL_PRI & IOL1WAY_OFF & OSCIOFNC_ON & FCKSM_CSDCMD & FNOSC_PRIPLL & PLL96MHZ_ON & PLLDIV_NODIV & IESO_ON)
        _CONFIG3(WPFP_WPFP0 & SOSCSEL_SOSC & WUTSEL_LEG & WPDIS_WPDIS & WPCFG_WPCFGDIS & WPEND_WPENDMEM)
        _CONFIG4(DSWDTPS_DSWDTPS3 & DSWDTOSC_LPRC & RTCOSC_SOSC & DSBOREN_OFF & DSWDTEN_OFF)
 
// *****************************************************************************

// Application States
typedef enum
{
    BT_INITIALIZE = 0,                // Initialize the app when a device is attached
    BT_STATE_IDLE,                    // Inactive State
    BT_STATE_PROCESS,
	BT_STATE_WRITE_CLASS,
	BT_STATE_READ_EP1,
	BT_STATE_READ_CLASS_WAITING,
	BT_STATE_READ_ACL,
	BT_STATE_WRITE_ACL,
	BT_STATE_READ_ACL_HCI,
	BT_STATE_READ_ACL_WAITING,
	BT_STATE_READ_HCI_WAITING,
	BT_STATE_READ_HCI,

    BT_STATE_ERROR                    // An error has occured
} BT_STATE;

// Hci States
typedef enum
{
	HCI_CMD_RESET = 0,                // Initialize the hci when a device is attached
		HCI_CMD_RESET_END,
	HCI_CMD_READ_BD_ADDR,
		HCI_CMD_READ_BD_ADDR_END,
	HCI_CMD_LOCAL_NAME,
		HCI_CMD_LOCAL_NAME_END,
	HCI_CMD_CLASS_DEVICE,
		HCI_CMD_CLASS_DEVICE_WRITE_END,
	HCI_CMD_SCAN_ENABLE,
		HCI_CMD_SCAN_ENABLE_WRITE_END,
	HCI_CMD_WAIT_CONNECTION,
	HCI_CMD_CONNECTION_ACCEPT,
		HCI_CMD_CONNECTION_ACCEPT_WRITE_END,
	HCI_CMD_PIN_CODE_WAIT,
		HCI_CMD_PIN_CODE_RESP,
		HCI_CMD_PIN_CODE_RESP_END,
	HCI_LINK_KEY_RESP,
	HCI_CMD_DISCONECT,

	L2CAP_CON_REQ,
	L2CAP_CON_RESP,
	L2CAP_CONFIG_REQ,
	L2CAP_CONFIG_RESP,
	L2CAP_CONFIG_REQ_HOST,
		L2CAP_CONFIG_RESP_HOST,
	L2CAP_DISCONNECT_REQ,
		L2CAP_DISCONNECT_REQ_END,

	SDP_SEARCH_REQ,
	SDP_SERCH_RESP,
	SDP_ATTR_RESP1,
		SDP_ATTR_RESP1_READ,
	SDP_ATTR_RESP2,
	SDP_ATTR_RESP3_INIT,
		SDP_ATTR_RESP3,
	SDP_ATTR_RESP4_READ,
	SDP_ATTR_END,

	RFCOMM_SABM03_READ,
		RFCOMM_UA03_RESP,
	RFCOMM_PN_READ,
	RFCOMM_PN_READ1,
		RFCOMM_PN_RESP,
	RFCOMM_SABM0b_READ,
		RFCOMM_UA0b_RESP,
	RFCOMM_MSC_READ1,
		RFCOMM_MSC_RESP1,
	RFCOMM_MSC_READ2,
		RFCOMM_MSC_HATENA,
		RFCOMM_MSC_RESP2,
	RFCOMM_MSC_READ3,
		RFCOMM_MSC_RESP3,
	RFCOMM_RPN_READ1,
		RFCOMM_RPN_RESP1,
	RFCOMM_RPN_READ2,
		RFCOMM_RPN_RESP2,
	RFCOMM_MSC_READ4,
		RFCOMM_MSC_RESP4,
	RFCOMM_DATA_READ,
	RRFCOMM_DATA_RECEIVED,
	RRFCOMM_DATA_WRITE,

	RRFCOMM_END1,
	RRFCOMM_END1_RESP,
	RRFCOMM_END2,
	RRFCOMM_END3,
	RRFCOMM_END3_RESP,

	L2CAP_END1,
	L2CAP_END1_RESP,
	L2CAP_END_REQ,
	L2CAP_END_REQ_RESP,

	PROG_END
} HCI_STATE;


// *****************************************************************************
// *****************************************************************************
// Global Variables
// *****************************************************************************
// *****************************************************************************


BYTE	deviceAddress;  // Address of the device on the USB
BT_STATE  DemoState;      // Current state of the demo application
HCI_STATE  HciState;      // Current state of the demo application
WORD data_size;
int data_num,length;
unsigned char	local_bd_addr[6];//this 
unsigned char	remote_bd_addr[6];//PC
unsigned char buf[64];
unsigned char buf1[64];
unsigned char link_key[64];
unsigned char handle[2];//a handle for ACL 
unsigned char src_cid[2];
unsigned char dst_cid[2];
unsigned char rfcomm_flag=0;
unsigned char EP2busy_flag=0;
unsigned char dlci;
unsigned char step=0;
int pdata;
char message[30];
int end_num;

//SDP for Microsoft
unsigned char att[4][31]=
{
{0x26,0x20,0x1b,0x00,0x3d,0x00,0xFF,0xFF,0x07,0xFF,0xFF,0x00,0x38,0x00,0x35,0x36,0x00,0x32,0x35,0x30,0x09,0x00,0x00,0x0a,0x00,0x01,0x00,0x00,0x09,0x00,0x01},
{0x26,0x10,0x1b,0x00,0x35,0x03,0x19,0x11,0x01,0x09,0x00,0x04,0x35,0x0c,0x35,0x03,0x19,0x01,0x00,0x35,0x05,0x19,0x00,0x03,0x08,0x01,0x09,0x01,0x00,0x25,0x0a},
{0x26,0x10,0x0b,0x00,0x42,0x4c,0x55,0x45,0x4d,0x45,0x5f,0x43,0x4f,0x4d,0x00,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF}
};


//******************************************************************************
//******************************************************************************
// Local Routines
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        InitializeSystem
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         TRUE if successful, FALSE if not.
 *
 * Side Effects:    See below
 *
 * Overview:        This routine initializes the processor and peripheral,
 *                  setting clock speeds and enabling any required
 *                  features.
 *************************************************************************/

BOOL InitializeSystem ( void )
{
	unsigned int pll_startup_counter = 600;
	CLKDIVbits.PLLEN = 1;
	while(pll_startup_counter--);

	// Configure U2RX - put on pin 17 (RP8)
	RPINR19bits.U2RXR = 8;
	// Configure U2TX - put on pin 16  (RP7)
	RPOR3bits.RP7R = 5;
    // Init UART
    UART2Init();

    DataEEInit();
    dataEEFlags.val = 0;
    // Set Default demo state
    DemoState = BT_INITIALIZE;
	HciState=HCI_CMD_RESET;
    return TRUE;
} // InitializeSystem


/*************************************************************************
 * Function:        CheckForNewAttach
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          deviceAddress (global)
 *                  Updates the device address when an attach is found.
 *
 * Returns:         TRUE if a new device has been attached.  FALSE,
 *                  otherwise.
 *
 * Side Effects:    Prints attach message
 *
 * Overview:        This routine checks to see if a new device has been
 *                  attached.  If it has, it records the address.
 *************************************************************************/

BOOL CheckForNewAttach ( void )
{
    // Try to get the device address, if we don't have one.
    if (deviceAddress == 0)
    {
        GENERIC_DEVICE_ID DevID;

        DevID.vid   = 0x04D8;
        DevID.pid   = 0x000C;
        #ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
            DevID.serialNumberLength = 0;
            DevID.serialNumber = NULL;
        #endif

        if (USBHostGenericGetDeviceAddress(&DevID))
        {
            deviceAddress = DevID.deviceAddress;
            UART2PrintString( "Generic demo device attached - polled, deviceAddress=" );
            UART2PutDec( deviceAddress );
            UART2PrintString( "\r\n" );
            return TRUE;
        }
    }

    return FALSE;

} // CheckForNewAttach


/*************************************************************************
 * Function:        ManageDemoState
 *
 * Preconditions:   The DemoState global variable must be initialized to
 *                  DEMO_STATE_IDLE (0).  (This occurs on reset.)
 *
 * Input:           DemoState (global)
 *                  Actions selected based value of DemoState on function
 *                  entry.
 *
 *                  deviceAddress (global)
 *                  May use device address to access device, depending on
 *                  state.
 *
 *                  DataPacket (global)
 *                  May read data from packet buffer, depending on state.
 *
 * Output:          DemoState (global)
 *                  Updates demo state as appropriate.
 *
 *                  DataPacket (global)
 *                  May cause data in the packet buffer to be updated,
 *                  depending on state.
 *
 * Returns:         None
 *
 * Side Effects:    Depend on state transition
 *
 * Overview:        This routine maintains the state of the application,
 *                  updateing global data and taking actions as necessary
 *                  to maintain the custom demo operations.
 *************************************************************************/
void ManageDemoState ( void )
{
    BYTE RetVal;

    // Watch for device detaching
    if (USBHostGenericDeviceDetached(deviceAddress) && deviceAddress != 0)
    {
        UART2PrintString( "Generic demo device detached - polled\r\n" );
        DemoState = BT_INITIALIZE;//YTS
        deviceAddress   = 0;
    }

    switch (DemoState)
    {
    case BT_INITIALIZE://YTS
//        InitLcdMessage();YTS
        DemoState = BT_STATE_IDLE;//YTS
        break;

    /** Idle State:  Loops here until attach **/
    case BT_STATE_IDLE://YTS
        if (CheckForNewAttach())
        {
			DemoState = BT_STATE_PROCESS;//YTS
			HciState = HCI_CMD_RESET;//YTS
        }
        break;

    case BT_STATE_PROCESS:
		switch (HciState)
		{
//HCI layer***********************************************************************
		case HCI_CMD_RESET:
			buf1[0]=0x03;
			buf1[1]=0x0c;
			buf1[2]=0;
			data_size=3;
			DemoState = BT_STATE_WRITE_CLASS;
			HciState = HCI_CMD_RESET_END;
		    break;

		case HCI_CMD_RESET_END:
			end_num=0x0e;// When buf1[0]==end_num, reading process ends. See BT_STATE_READ_EP1
			strcpy(message,"HCI_CMD_RESET: ");//message for BT_STATE_READ_EP1
			DemoState = BT_STATE_READ_EP1;
			HciState = HCI_CMD_READ_BD_ADDR;
		    break;

//********************************************************************************
		case HCI_CMD_READ_BD_ADDR:
			buf1[0]=0x09;
			buf1[1]=0x10;
			buf1[2]=0;
			data_size=3;
			DemoState = BT_STATE_WRITE_CLASS;
			HciState = HCI_CMD_READ_BD_ADDR_END;
			break;

		case HCI_CMD_READ_BD_ADDR_END:
			end_num=0x0e;strcpy(message,"HCI_CMD_READ_BD_ADDR: ");
			DemoState = BT_STATE_READ_EP1;
			HciState = HCI_CMD_LOCAL_NAME;
		    break;

//********************************************************************************
		case HCI_CMD_LOCAL_NAME:
//local_bd_addr[6];
			buf1[0]=0x13;
			buf1[1]=0x0c;
			buf1[2]=0x04;
			buf1[3]='y';
			buf1[4]='t';
			buf1[5]='s';
			buf1[6]=0x00;
			data_size=7;
			DemoState = BT_STATE_WRITE_CLASS;
			HciState = HCI_CMD_LOCAL_NAME_END;
			break;

		case HCI_CMD_LOCAL_NAME_END:
			end_num=0x0e;strcpy(message,"HCI_CMD_LOCAL_NAME: ");
			DemoState = BT_STATE_READ_EP1;
			HciState = HCI_CMD_CLASS_DEVICE;
		    break;

//********************************************************************************
		case HCI_CMD_CLASS_DEVICE:
			buf1[0]=0x24;
			buf1[1]=0x0c;
			buf1[2]=0x03;
			buf1[3]=0x12;//
			buf1[4]=0x01;//
			buf1[5]=0x00;

			data_size=6;
			DemoState = BT_STATE_WRITE_CLASS;
			HciState = HCI_CMD_CLASS_DEVICE_WRITE_END;
			break;

		case HCI_CMD_CLASS_DEVICE_WRITE_END:
			end_num=0x0e;strcpy(message,"HCI_CMD_CLASS_DEVICE: ");
			DemoState = BT_STATE_READ_EP1;
			HciState = HCI_CMD_SCAN_ENABLE;
		    break;

//********************************************************************************
		case HCI_CMD_SCAN_ENABLE:
			buf1[0]=0x1a;
			buf1[1]=0x0c;
			buf1[2]=0x01;
			buf1[3]=0x03;
			data_size=4;
			DemoState = BT_STATE_WRITE_CLASS;
			HciState = HCI_CMD_SCAN_ENABLE_WRITE_END;
			break;

		case HCI_CMD_SCAN_ENABLE_WRITE_END:
			end_num=0x0e;strcpy(message,"HCI_CMD_SCAN_ENABLE: ");
			DemoState = BT_STATE_READ_EP1;
			HciState = HCI_CMD_WAIT_CONNECTION;
		    break;

//********************************************************************************
		case HCI_CMD_WAIT_CONNECTION:
			end_num=0x04;strcpy(message,"HCI_CMD_WAIT_CONNECTION: ");
			DemoState = BT_STATE_READ_EP1;
			HciState = HCI_CMD_CONNECTION_ACCEPT;
rfcomm_flag=0;
			break;

//********************************************************************************
		case HCI_CMD_CONNECTION_ACCEPT:
			buf1[8]=buf1[7];//copy address
			buf1[7]=buf1[6];
			buf1[6]=buf1[5];
			buf1[5]=buf1[4];
			buf1[4]=buf1[3];
			buf1[3]=buf1[2];
			buf1[0]=0x09;
			buf1[1]=0x04;
			buf1[2]=0x07;
			buf1[9]=0x01;//role
			data_size=10;
			DemoState = BT_STATE_WRITE_CLASS;
			HciState = HCI_CMD_CONNECTION_ACCEPT_WRITE_END;
			break;

		case HCI_CMD_CONNECTION_ACCEPT_WRITE_END:
			end_num=0x03;strcpy(message,"HCI_CMD_CONNECTION_ACCEPT: ");
			DemoState = BT_STATE_READ_EP1;
			HciState = L2CAP_CON_REQ;
		    break;

//L2CAP layer*********************************************************************
		case  L2CAP_CON_REQ:
			handle[0]=buf1[3];handle[1]=buf1[4]+0x20;

			strcpy(message,"L2CAP_CON_REQ \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = L2CAP_CON_RESP;
			break;

		case  L2CAP_CON_RESP:
			if (buf[12]==0x03) 	rfcomm_flag=2;//RFCOMM CHANNEL OPEN STATUS

			buf[2]=0x10;
			buf[4]=0x0c;
			buf[8]=0x03;//
			buf[10]=0x08;
			buf[12]=buf[14]+4;//dst_cid (PIC)
			buf[13]=buf[15];//
			buf[16]=0x00;//result
			buf[17]=0x00;//
			buf[18]=0x00;//statis
			buf[19]=0x00;//
			data_size=20;

			src_cid[0]=buf[14];	src_cid[1]=buf[15];		
			dst_cid[0]=buf[12];	dst_cid[1]=buf[13];

			DemoState = BT_STATE_WRITE_ACL;
			HciState = L2CAP_CONFIG_REQ;
			break;
//********************************************************************************
		case  L2CAP_CONFIG_REQ:
			strcpy(message,"L2CAP_CONFIG_REQ \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = L2CAP_CONFIG_RESP;
			break;

		case  L2CAP_CONFIG_RESP:
			buf[18]=0x01;//buf[16];//option
			buf[19]=0x02;//buf[17];//
			buf[20]=0x00;//buf[18];//
			buf[21]=0x10;//buf[19];//
			buf[2]=0x12;
			buf[4]=0x0e;
			buf[8]=0x05;//
			buf[10]=0x0a;
			buf[12]=src_cid[0];
			buf[13]=src_cid[1];
			buf[16]=0x00;//result
			buf[17]=0x00;//
			data_size=22;
			DemoState = BT_STATE_WRITE_ACL;
			HciState = L2CAP_CONFIG_REQ_HOST;
			break;

//********************************************************************************
		case  L2CAP_CONFIG_REQ_HOST:
			buf[16]=0x01;//buf[18];//option
			buf[17]=0x02;//buf[19];//
			buf[18]=0x00;//buf[20];//
			buf[19]=0x10;//buf[21];//
			buf[2]=0x10;
			buf[4]=0x0c;
			buf[8]=0x04;//
			buf[10]=0x08;
			buf[12]=src_cid[0];
			buf[13]=src_cid[1];
			buf[14]=0x00;//result
			buf[15]=0x00;//
			data_size=20;
			DemoState = BT_STATE_WRITE_ACL;
			HciState = L2CAP_CONFIG_RESP_HOST;
			break;

//********************************************************************************
		case  L2CAP_CONFIG_RESP_HOST:
			strcpy(message,"L2CAP_CONFIG_REQ_HOST \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = SDP_SEARCH_REQ;
			break;

//SDP layer **********************************************************************
		case  SDP_SEARCH_REQ:
			if(rfcomm_flag==2) {HciState =RFCOMM_SABM03_READ;break;}//GO TO RFCOMM LAYER

			strcpy(message,"SDP_SEARCH_REQ \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = SDP_SERCH_RESP;
			break;

//********************************************************************************
		case  SDP_SERCH_RESP:

			if(buf[16]==0x11) rfcomm_flag=1;
			//SDP data initialization
			for(step=0;step<3;step++){att[step][0]=buf[0];}
			att[0][6]=src_cid[0]; att[0][7]=src_cid[1];
			att[0][9]=buf[9]; att[0][10]=buf[10];
			HciState = SDP_ATTR_RESP1;

			step=0;
			break;

//Microsoft ***********************************************************************
		case  SDP_ATTR_RESP1:
		if(step==3) {HciState = SDP_ATTR_RESP1_READ;step=0;break;}	
			data_size=att[step][2]+4;
			for(pdata=0;pdata<data_size;pdata++) 	buf[pdata]=att[step][pdata];
			step++; 

			DemoState = BT_STATE_WRITE_ACL;
			break;

//********************************************************************************
		case  SDP_ATTR_RESP1_READ:
			strcpy(message,"SDP_ATTR_RESP1_READ \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			if(rfcomm_flag==0) HciState = SDP_ATTR_RESP2;
			else HciState = SDP_ATTR_RESP3_INIT;//rfcomm_flag==1
			break;

//********************************************************************************
		case  SDP_ATTR_RESP2:

//		buf[0]=0x26;
//		buf[1]=0x20;

			buf[2]=0x0e;
			buf[3]=0x00;
			buf[4]=0x0a;
			buf[5]=0x00;
			buf[6]=src_cid[0];
			buf[7]=src_cid[1];
			buf[8]=0x07;
			buf[9]=0x00;
//			buf[10]=0x00;
//			buf[11]=0x00;
			buf[12]=0x05;
			buf[13]=0x00;
			buf[14]=0x02;
			buf[15]=0x35;
			buf[16]=0x00;
			buf[17]=0x00;
			data_size=18;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = SDP_ATTR_RESP4_READ;
			break;

//********************************************************************************
		case  SDP_ATTR_RESP3_INIT:

				//SDP data initialization
				for(step=0;step<3;step++){att[step][0]=buf[0];}
				att[0][6]=src_cid[0]; att[0][7]=src_cid[1];
				att[0][9]=buf[9]; att[0][10]=buf[10];
				step=0;

			HciState = SDP_ATTR_RESP3;
			break;

//********************************************************************************
		case  SDP_ATTR_RESP3:
		if(step==3) {HciState = SDP_ATTR_RESP4_READ;step=0;break;}
	
			data_size=att[step][2]+4;
			for(pdata=0;pdata<data_size;pdata++) 	buf[pdata]=att[step][pdata];
			step++; 

			DemoState = BT_STATE_WRITE_ACL;
			break;

//********************************************************************************
		case  SDP_ATTR_RESP4_READ:
			strcpy(message,"3_SDP_ATTR_RESP \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = SDP_ATTR_END;
			break;

//********************************************************************************
		case  SDP_ATTR_END:
            buf[8]=0x07;
            data_size=16;
			DemoState = BT_STATE_WRITE_ACL;
			HciState = L2CAP_CON_REQ;
		break;

//********************************************************************************
		case L2CAP_DISCONNECT_REQ:
			buf1[0]=0x06;
			buf1[1]=0x04;
			buf1[2]=0x03;
			buf1[3]=handle[0];
			buf1[4]=handle[1]-0x20;
			buf1[5]=0x13;

			DemoState = BT_STATE_WRITE_CLASS;
			HciState = L2CAP_DISCONNECT_REQ_END;
			break;

		case L2CAP_DISCONNECT_REQ_END:
			#ifdef DEBUG_MODE
            UART2PrintString( "HCI_DISCONNECT !\r\n" );
			#endif

			HciState = HCI_CMD_WAIT_CONNECTION;
		break;

//RFCOMM**************************************************************************
		case  RFCOMM_SABM03_READ:
			strcpy(message,"RFCOMM_SABM03_READ \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = RFCOMM_UA03_RESP;
			break;

//********************************************************************************
		case  RFCOMM_UA03_RESP:
//		buf[0]=0x26;
//		buf[1]=0x20;

			buf[2]=0x08;
			buf[3]=0x00;
			buf[4]=0x04;
			buf[5]=0x00;
			buf[6]=src_cid[0];
			buf[7]=src_cid[1];
			buf[8]=0x03;
			buf[9]=0x73;
			buf[10]=0x01;
			buf[11]=0xd7;
			data_size=12;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = HCI_CMD_PIN_CODE_WAIT;
//HciState = RFCOMM_PN_READ;
			break;

//********************************************************************************
		case HCI_CMD_PIN_CODE_WAIT:

			end_num=0x0e;strcpy(message,"HCI_CMD_WAIT_PIN_CODE: ");
			DemoState = BT_STATE_READ_EP1;
			HciState = HCI_CMD_PIN_CODE_RESP;
		    break;

//********************************************************************************
		case HCI_CMD_PIN_CODE_RESP:
			if(buf1[0]==0x17){
				buf1[0]=0x0b;
				buf1[1]=0x04;
				buf1[2]=0x16;
				for(step=0;step<22;step++) buf1[3+step] = DataEERead(1+step);
				data_size=25;

				#ifdef DEBUG_MODE
				UART2PrintString( "HCL: " );
				for(data_num=0;data_num<buf1[2]+4;data_num++)
      	 	         {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
				UART2PrintString( "\r\n" );
				#endif

				DemoState = BT_STATE_WRITE_CLASS;
				HciState = HCI_CMD_PIN_CODE_RESP_END;
				break;
			}

			buf1[0]=0x0d;
			buf1[1]=0x04;

			buf1[8]=buf1[7];//copy address
			buf1[7]=buf1[6];
			buf1[6]=buf1[5];
			buf1[5]=buf1[4];
			buf1[4]=buf1[3];
			buf1[3]=buf1[2];
			buf1[2]=0x17;//length
			buf1[9]=0x04;
			buf1[10]='0';
			buf1[11]='0';
			buf1[12]='0';
			buf1[13]='0';
			for(step=14;step<12;step++) buf1[step]=0;

			data_size=26;

			#ifdef DEBUG_MODE
			UART2PrintString( "HCL: " );
			for(data_num=0;data_num<buf1[2]+2;data_num++)
      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			DemoState = BT_STATE_WRITE_CLASS;
			HciState = HCI_CMD_PIN_CODE_RESP_END;
			break;

//********************************************************************************
		case HCI_CMD_PIN_CODE_RESP_END:
			end_num=0x0e;strcpy(message,"HCI_CMD_PIN_CODE_RESP: ");
			DemoState = BT_STATE_READ_EP1;
			HciState = RFCOMM_PN_READ;
		    break;

//********************************************************************************
		case RFCOMM_PN_READ:
			strcpy(message,"RFCOMM_PN_READ \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = RFCOMM_PN_READ1;
			break;

//********************************************************************************
		case RFCOMM_PN_READ1://For Windows 7
			if(buf[6]!=0x01) {HciState = RFCOMM_PN_RESP;break;}
			strcpy(message,"RFCOMM_PN_READ1 \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = RFCOMM_PN_RESP;
			break;

//********************************************************************************
		case  RFCOMM_PN_RESP:
//		buf[0]=0x26;
//		buf[1]=0x20;

			buf[2]=0x12;
			buf[3]=0x00;
			buf[4]=0x0e;
			buf[5]=0x00;
			buf[6]=src_cid[0];
			buf[7]=src_cid[1];
			buf[8]=0x01;
			buf[9]=0xef;
//			buf[10]=0x15;
			buf[11]=0x81;
//			buf[12]=0x11;

			buf[14]=0xE0;
//			buf[15]=0x07;

//			buf[17]=0x9A;
//			buf[18]=0x02;


//			buf[20]=0x07;
			buf[21]=0xAA;
			data_size=22;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = RFCOMM_SABM0b_READ;
			break;

//********************************************************************************
		case RFCOMM_SABM0b_READ:
			strcpy(message,"RFCOMM_SABM0b_READ \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = RFCOMM_UA0b_RESP;
			break;


//********************************************************************************
		case HCI_LINK_KEY_RESP:
dlci=buf[8];
			buf1[0]=0x0b;
			buf1[1]=0x04;
			buf1[2]=0x16;
			for(data_num=0;data_num<0x16;data_num++) buf1[3+data_num]=link_key[3+data_num];
			

			#ifdef DEBUG_MODE
			UART2PrintString( "LINK_KEY: " );
			for(data_num=0;data_num<buf1[2]+3;data_num++)
      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif
			data_size=buf1[2]+3;

			DemoState = BT_STATE_WRITE_CLASS;
			HciState = RFCOMM_UA0b_RESP;
			break;


//********************************************************************************
		case  RFCOMM_UA0b_RESP:
			dlci=buf[8];
//		buf[0]=0x26;
//		buf[1]=0x20;

			buf[2]=0x08;
			buf[3]=0x00;
			buf[4]=0x04;
			buf[5]=0x00;
			buf[6]=src_cid[0];
			buf[7]=src_cid[1];
			buf[8]=dlci;
			buf[9]=0x73;
			buf[10]=0x01;
			buf[11]=0x92;
			data_size=12;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = RFCOMM_MSC_READ1;
			break;
	
			
//********************************************************************************
		case RFCOMM_MSC_READ1:
			strcpy(message,"RFCOMM_MSC_READ1 \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = RFCOMM_MSC_RESP1;
			break;			
			

//********************************************************************************
		case  RFCOMM_MSC_RESP1:

//		buf[0]=0x26;
//		buf[1]=0x20;

			buf[2]=0x0c;
			buf[3]=0x00;
			buf[4]=0x08;//0x08;
			buf[5]=0x00;
			buf[6]=src_cid[0];
			buf[7]=src_cid[1];
			buf[8]=0x01;
			buf[9]=0xef;
			buf[10]=0x09;
			buf[11]=0xe3;//0xe3;
			buf[12]=0x05;
			buf[13]=dlci;
			buf[14]=0x8d;
			buf[15]=0xaa;
			data_size=16;

			DemoState = BT_STATE_WRITE_ACL;
			HciState =  RFCOMM_MSC_READ2;
			break;

//********************************************************************************
		case RFCOMM_MSC_READ2:
			strcpy(message,"RFCOMM_MSC_READ2 \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = RFCOMM_MSC_HATENA;
			break;

//********************************************************************************
		case  RFCOMM_MSC_HATENA:
//			dlci=buf[8];
//		buf[0]=0x26;
//		buf[1]=0x20;

			buf[2]=0x09;
			buf[3]=0x00;
			buf[4]=0x05;
			buf[5]=0x00;
			buf[6]=src_cid[0];
			buf[7]=src_cid[1];
			buf[8]=dlci-2;//0x21;
			buf[9]=0xff;
			buf[10]=0x01;
			buf[11]=0x7f;
			buf[12]=0x9c;
			data_size=13;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = RFCOMM_MSC_RESP2;
			break;


//********************************************************************************
		case  RFCOMM_MSC_RESP2:
//		buf[0]=0x26;
//		buf[1]=0x20;

			buf[2]=0x0c;
			buf[3]=0x00;
			buf[4]=0x08;
			buf[5]=0x00;
			buf[6]=src_cid[0];
			buf[7]=src_cid[1];
			buf[8]=0x01;
			buf[9]=0xef;
			buf[10]=0x09;
			buf[11]=0xe1;
			buf[12]=0x05;
			buf[13]=dlci;
			buf[14]=0x8d;
			buf[15]=0xaa;
			data_size=16;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = RFCOMM_MSC_READ3;
			break;


//********************************************************************************
		case RFCOMM_MSC_READ3:
			strcpy(message,"RFCOMM_MSC_READ3 \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = RFCOMM_MSC_RESP3;
			break;

//********************************************************************************
		case  RFCOMM_MSC_RESP3:
//		buf[0]=0x26;
//		buf[1]=0x20;

			buf[2]=0x0c;
			buf[3]=0x00;
			buf[4]=0x08;
			buf[5]=0x00;
			buf[6]=src_cid[0];
			buf[7]=src_cid[1];
			buf[8]=0x01;
			buf[9]=0xef;
			buf[10]=0x09;
			buf[11]=0xe1;
			buf[12]=0x05;
			buf[13]=dlci;
			buf[14]=0x8d;
			buf[15]=0xaa;
			data_size=16;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = RFCOMM_RPN_READ1;
			break;

//********************************************************************************
		case RFCOMM_RPN_READ1:
			strcpy(message,"RFCOMM_RPN_READ1 \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = RFCOMM_RPN_RESP1;
			break;

//********************************************************************************
		case  RFCOMM_RPN_RESP1:
//		buf[0]=0x26;
//		buf[1]=0x20;

			buf[2]=0x12;
			buf[3]=0x00;
			buf[4]=0x0e;
			buf[5]=0x00;
			buf[6]=src_cid[0];
			buf[7]=src_cid[1];
			buf[8]=0x01;
			buf[9]=0xef;
			buf[10]=0x15;
			buf[11]=0x91;
			buf[12]=0x11;
			buf[13]=dlci;
			buf[14]=0x03;
			buf[15]=0x00;
			buf[16]=0x00;
			buf[17]=0x00;
			buf[18]=0x00;
			buf[19]=0x01;
			buf[20]=0x00;
			buf[21]=0xaa;
			data_size=22;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = RFCOMM_RPN_READ2;
			break;

//********************************************************************************
		case RFCOMM_RPN_READ2:
			strcpy(message,"RFCOMM_RPN_READ2 \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = RFCOMM_RPN_RESP2;
			break;

//********************************************************************************
		case  RFCOMM_RPN_RESP2:
//		buf[0]=0x26;
//		buf[1]=0x20;

			buf[2]=0x12;
			buf[3]=0x00;
			buf[4]=0x0e;
			buf[5]=0x00;
			buf[6]=src_cid[0];
			buf[7]=src_cid[1];
			buf[8]=0x01;
			buf[9]=0xef;
			buf[10]=0x15;
			buf[11]=0x91;
			buf[12]=0x11;
			buf[13]=dlci;
			buf[14]=0x03;
			buf[15]=0x00;
			buf[16]=0x00;
			buf[17]=0x00;
			buf[18]=0x00;
			buf[19]=0x01;
			buf[20]=0x00;
			buf[21]=0xaa;
			data_size=22;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = RFCOMM_MSC_READ4;
			break;

//********************************************************************************
		case RFCOMM_MSC_READ4:
			strcpy(message,"RFCOMM_MSC_READ4 \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = RFCOMM_MSC_RESP4;
			break;

//********************************************************************************
		case  RFCOMM_MSC_RESP4:
//		buf[0]=0x26;
//		buf[1]=0x20;

			buf[2]=0x0c;
			buf[3]=0x00;
			buf[4]=0x08;
			buf[5]=0x00;
			buf[6]=src_cid[0];
			buf[7]=src_cid[1];
			buf[8]=0x01;
			buf[9]=0xef;
			buf[10]=0x09;
			buf[11]=0xe1;
			buf[12]=0x05;
			buf[13]=dlci;
			buf[14]=0x09;
			buf[15]=0xaa;
			data_size=16;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = RFCOMM_DATA_READ;
			break;


//********************************************************************************
//********************************************************************************
		case RFCOMM_DATA_READ:
			strcpy(message,"RFCOMM_DATA \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = RRFCOMM_DATA_RECEIVED;
			break;

//********************************************************************************
		case  RRFCOMM_DATA_RECEIVED:
if(buf[9]==0x53) {HciState = RRFCOMM_END1;break;}
			length=buf[4]-1;
			buf1[1]=buf[12];//copy of received data

			buf[0]=handle[0];//connection handle+ PB flag + BC flag 
			buf[1]=handle[1];

			buf[2]=0x09;
			buf[3]=0x00;
			buf[4]=0x05;
			buf[5]=0x00;
			buf[6]=src_cid[0];
			buf[7]=src_cid[1];
			buf[8]=0x09;//address
			buf[9]=0xff;//UIH
			buf[10]=0x01;//length=0
			buf[11]=0x1a;//FC off
			buf[12]=0x5c;
			data_size=13;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = RRFCOMM_DATA_WRITE;

			break;

//********************************************************************************
		case  RRFCOMM_DATA_WRITE:
			if(buf[9]==0x53) {HciState =L2CAP_CON_REQ;rfcomm_flag=1;break;}

//			buf[0]=handle[0];//connection handle+ PB flag + BC flag 
//			buf[1]=handle[1];

			buf[2]=length+4;
			buf[3]=0x00;
			buf[4]=length;
			buf[5]=0x00;
			buf[6]=src_cid[0];
			buf[7]=src_cid[1];
			buf[8]=dlci-2;//0x21;//dlci;//0x0b;
			buf[9]=0xef;
			buf[10]=((length-4) <<1) +1;
			buf[11]=buf1[1]+1;//top of data
			data_size=buf[2]+4;;
			for(step=0;step<(length-4);step++) buf[12+step]=buf[13+step];//copy data

			buf[data_size-1]=0x40;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = RFCOMM_DATA_READ;

			break;

//********************************************************************************
//********************************************************************************
		case  RRFCOMM_END1:


//			buf[0]=handle[0];//connection handle+ PB flag + BC flag 
//			buf[1]=handle[1];
//			buf[2]=0x09;
//			buf[3]=0x00;
//			buf[4]=0x05;
//			buf[5]=0x00;
			buf[6]=src_cid[0];
			buf[7]=src_cid[1];
//			buf[8]=0x09;//
			buf[9]=0x73;//
			buf[10]=0x01;//
			buf[11]=0x92;//
			data_size=12;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = RRFCOMM_END3;
			break;
//********************************************************************************
		case  RRFCOMM_END3:

//			buf[0]=handle[0];//connection handle+ PB flag + BC flag 
//			buf[1]=handle[1];
//			buf[2]=0x09;
//			buf[3]=0x00;
//			buf[4]=0x05;
//			buf[5]=0x00;
			buf[6]=src_cid[0];
			buf[7]=src_cid[1];
			buf[8]=0x01;//
			buf[9]=0x53;//
			buf[10]=0x01;//
			buf[11]=0x9c;//
			data_size=12;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = RRFCOMM_END1_RESP;
			break;
//********************************************************************************
		case  RRFCOMM_END1_RESP:
			strcpy(message,"RRFCOMM_END1_RESP \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = RRFCOMM_END2;
			break;
//********************************************************************************
		case  RRFCOMM_END2:

//			buf[0]=handle[0];//connection handle+ PB flag + BC flag 
//			buf[1]=handle[1];
//			buf[2]=0x09;
//			buf[3]=0x00;
//			buf[4]=0x05;
//			buf[5]=0x00;
			buf[6]=src_cid[0];
			buf[7]=src_cid[1];
//			buf[8]=0x09;//
			buf[9]=0x73;//
			buf[10]=0x01;//
			buf[11]=0xd7;//
			data_size=12;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = RRFCOMM_END3_RESP;
			break;


//********************************************************************************
		case  RRFCOMM_END3_RESP:
			strcpy(message,"RRFCOMM_END3_RESP \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = L2CAP_END1;
			break;


//********************************************************************************
		case  L2CAP_END1:
			strcpy(message,"L2CAP_END1 \r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = PROG_END;
			HciState = L2CAP_END1_RESP;
			break;
//********************************************************************************
		case  L2CAP_END1_RESP:
			buf[8]=0x06;//
//			buf[9]=0x42;//??????
			buf[12]=src_cid[0];
			buf[13]=src_cid[1];
			buf[14]=dst_cid[0];
			buf[15]=dst_cid[1];

			data_size=16;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = L2CAP_END_REQ;
			break;
//********************************************************************************
		case  L2CAP_END_REQ:

			buf[8]=0x07;//
//			buf[9]=0x42;//??????
			buf[12]=dst_cid[0];
			buf[13]=dst_cid[1];
			buf[14]=src_cid[0];
			buf[15]=src_cid[1];

			data_size=16;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = L2CAP_END_REQ_RESP;
			break;
//********************************************************************************
		case  L2CAP_END_REQ_RESP:
			strcpy(message,"L2CAP_END_REQ_RESP\r\n");
			DemoState = BT_STATE_READ_ACL_HCI;
			HciState = PROG_END;
			HciState = HCI_CMD_DISCONECT;
			break;

//********************************************************************************
		case HCI_CMD_DISCONECT:
			buf1[0]=0x06;
			buf1[1]=0x04;
			buf1[2]=0x03;
			buf1[3]=buf[0];
			buf1[4]=buf[1]& 0xdf;
			buf1[5]=0;
			data_size=6;

			#ifdef DEBUG_MODE
			UART2PrintString("HCI_CMD_DISCONECT:");
			for(data_num=0;data_num<6;data_num++)
      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			DemoState = BT_STATE_WRITE_CLASS;
			HciState = HCI_CMD_SCAN_ENABLE;
			break;

//********************************************************************************
		case  PROG_END:
			break;

		}
		break;

//********************************************************************************
//@READ-WRITE FUNCTIONS
//********************************************************************************
//WRITE ENDPOINT 0
    case BT_STATE_WRITE_CLASS:
        if (!USBHostGenericTxIsBusy(deviceAddress))
		{
            if ( (RetVal=USBHostGenericClassRequest( deviceAddress, buf1, data_size )) == USB_SUCCESS )
            {
			//UART2PrintString( "HCI COMMAND SENT\r\n" );	
            DemoState = BT_STATE_PROCESS;
            }
            else
            {
			UART2PrintString( "Write Class Error !\r\n" );	
            }
        }
        break;


//READ ENDPOINT 1
    case BT_STATE_READ_EP1:
        if (!USBHostGenericRx1IsBusy(deviceAddress))
        {
            if ( (RetVal=USBHostGenericRead(deviceAddress, buf1, DATA_PACKET_LENGTH)) == USB_SUCCESS )
            {
                DemoState = BT_STATE_READ_CLASS_WAITING;
                //UART2PrintString( "READ EP1\r\n" );
            }
            else
            {
                UART2PrintString( "Device Read Error !\r\n" );
            }
        }
        break;

    case BT_STATE_READ_CLASS_WAITING:
        if (!USBHostGenericRx1IsBusy(deviceAddress)){

			if(buf1[0]!=end_num && buf1[0]!=0x16 && buf1[0]!=0x17){ DemoState =BT_STATE_READ_EP1; break;}

			#ifdef DEBUG_MODE
			UART2PrintString(message);
			for(data_num=0;data_num<buf1[1]+2;data_num++)
      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif
           DemoState = BT_STATE_PROCESS;
		}
        break;


//WRITE ENDPOINT 2
    case BT_STATE_WRITE_ACL:

			#ifdef DEBUG_MODE
			for(data_num=0;data_num<buf[2]+4;data_num++)
	        {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

        if (!USBHostGenericTxIsBusy(deviceAddress))
		{
            if ( (RetVal=USBHostGenericAclWrite( deviceAddress, buf, data_size )) == USB_SUCCESS )
            {
			//UART2PrintString( "HCI COMMAND SENT\r\n" );	
            DemoState = BT_STATE_PROCESS;
            }
            else
            {
			UART2PrintString( "Write Acl Error !\r\n" );	
            }
        }
        break;

//READ ENDPOINT 2 & 1 
    case BT_STATE_READ_ACL_HCI:
		if(EP2busy_flag==1) {DemoState = BT_STATE_READ_ACL_WAITING;break;}
        if (!USBHostGenericRx2IsBusy(deviceAddress))
        {
            if ( (RetVal=USBHostGenericAclRead(deviceAddress, buf, DATA_PACKET_LENGTH)) == USB_SUCCESS )
            {
                DemoState = BT_STATE_READ_ACL_WAITING;
            }
            else
            {
                UART2PrintString( "Read Acl Error !\r\n" );
            }
        }
        break;

    case BT_STATE_READ_ACL_WAITING:
        if (!USBHostGenericRx2IsBusy(deviceAddress)){

			#ifdef DEBUG_MODE
			UART2PrintString(message);
			for(data_num=0;data_num<buf[2]+4;data_num++)
   	  	        {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif
			EP2busy_flag=0;			
           DemoState = BT_STATE_READ_HCI;
		}
		else if(rfcomm_flag!=2){
			EP2busy_flag=1;
			DemoState = BT_STATE_READ_HCI;
		}
        break;

	case BT_STATE_READ_HCI:
		if(EP2busy_flag==0) buf1[0]=0xff;
        if (!USBHostGenericRx1IsBusy(deviceAddress))
        {
            if ( (RetVal=USBHostGenericRead(deviceAddress, buf1, DATA_PACKET_LENGTH)) == USB_SUCCESS )
            {
                DemoState = BT_STATE_READ_HCI_WAITING;
                //UART2PrintString( "READ EP1\r\n" );
            }
            else
            {
                UART2PrintString( "Device Read Error !\r\n" );
            }
        }
        break;

    case BT_STATE_READ_HCI_WAITING:
        if (!USBHostGenericRx1IsBusy(deviceAddress)){

			#ifdef DEBUG_MODE
			UART2PrintString( "HCI: " );
			for(data_num=0;data_num<buf1[1]+2;data_num++)
      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			if(EP2busy_flag==0){
				if(buf1[0]==0xff) {DemoState = BT_STATE_PROCESS;}
				else if(buf1[0]==0x18)  {
				for(step=0;step<22;step++)   DataEEWrite(buf1[step+2],step+1);

					DemoState = BT_STATE_READ_HCI;

				}
				else {DemoState = BT_STATE_READ_HCI;}
			}
			else{
				if(buf1[0]==0x05){HciState = L2CAP_DISCONNECT_REQ;DemoState = BT_STATE_PROCESS;}
				else {DemoState = BT_STATE_READ_ACL_WAITING;}
			}
		}
        break;

//********************************************************************************

    /** Error state:  Hold here until detached **/
    case BT_STATE_ERROR:                          // To Do: Flash LEDs
        break;

    default:
        DemoState = BT_INITIALIZE;
        break;
    }

    DelayMs(1); // 1ms delay

} // ManageDemoState

//******************************************************************************
//******************************************************************************
// USB Support Functions
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        USB_ApplicationEventHandler
 *
 * Preconditions:   The USB must be initialized.
 *
 * Input:           event       Identifies the bus event that occured
 *
 *                  data        Pointer to event-specific data
 *
 *                  size        Size of the event-specific data
 *
 * Output:          deviceAddress (global)
 *                  Updates device address when an attach or detach occurs.
 *
 *                  DemoState (global)
 *                  Updates the demo state as appropriate when events occur.
 *
 * Returns:         TRUE if the event was handled, FALSE if not
 *
 * Side Effects:    Event-specific actions have been taken.
 *
 * Overview:        This routine is called by the Host layer or client
 *                  driver to notify the application of events that occur.
 *                  If the event is recognized, it is handled and the
 *                  routine returns TRUE.  Otherwise, it is ignored (or
 *                  just "sniffed" and the routine returns FALSE.
 *************************************************************************/

BOOL USB_ApplicationEventHandler ( BYTE address, USB_EVENT event, void *data, DWORD size )
{
    #ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
        BYTE i;
    #endif

    // Handle specific events.
    switch (event)
    {
        case EVENT_GENERIC_ATTACH:
            if (size == sizeof(GENERIC_DEVICE_ID))
            {
                deviceAddress   = ((GENERIC_DEVICE_ID *)data)->deviceAddress;
                DemoState = BT_STATE_PROCESS; HciState=HCI_CMD_RESET;//YTS !!!!!!!!!!!!!!!!!
                UART2PrintString( "Generic demo device attached - event, deviceAddress=" );
                UART2PutDec( deviceAddress );
                UART2PrintString( "\r\n" );
                #ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
                    for (i=1; i<((GENERIC_DEVICE_ID *)data)->serialNumberLength; i++)
                    {
                        UART2PutChar( ((GENERIC_DEVICE_ID *)data)->serialNumber[i] );
                    }
                #endif
                UART2PrintString( "\r\n" );
                return TRUE;
            }
            break;

        case EVENT_GENERIC_DETACH:
            deviceAddress   = 0;
            DemoState = BT_INITIALIZE;//YTS
            UART2PrintString( "Generic demo device detached - event\r\n" );
            return TRUE;

        case EVENT_GENERIC_TX_DONE:           // The main state machine will poll the driver.
        case EVENT_GENERIC_RX1_DONE://YTS
        case EVENT_GENERIC_RX2_DONE://YTS
            return TRUE;

        case EVENT_VBUS_REQUEST_POWER:
            // We'll let anything attach.
            return TRUE;

        case EVENT_VBUS_RELEASE_POWER:
            // We aren't keeping track of power.
            return TRUE;

        case EVENT_HUB_ATTACH:
            UART2PrintString( "\r\n***** USB Error - hubs are not supported *****\r\n" );
            return TRUE;
            break;

        case EVENT_UNSUPPORTED_DEVICE:
            UART2PrintString( "\r\n***** USB Error - device is not supported *****\r\n" );
            return TRUE;
            break;

        case EVENT_CANNOT_ENUMERATE:
            UART2PrintString( "\r\n***** USB Error - cannot enumerate device *****\r\n" );
            return TRUE;
            break;

        case EVENT_CLIENT_INIT_ERROR:
            UART2PrintString( "\r\n***** USB Error - client driver initialization error *****\r\n" );
            return TRUE;
            break;

        case EVENT_OUT_OF_MEMORY:
            UART2PrintString( "\r\n***** USB Error - out of heap memory *****\r\n" );
            return TRUE;
            break;

        case EVENT_UNSPECIFIED_ERROR:   // This should never be generated.
            UART2PrintString( "\r\n***** USB Error - unspecified *****\r\n" );
            return TRUE;
            break;

        case EVENT_SUSPEND:
        case EVENT_DETACH:
        case EVENT_RESUME:
        case EVENT_BUS_ERROR:
            return TRUE;
            break;

        default:
            break;
    }

    return FALSE;

} // USB_ApplicationEventHandler


//******************************************************************************
//******************************************************************************
// Main
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        main
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         Never exits
 *
 * Side Effects:    Runs the application
 *
 * Overview:        This is the USB Custom Demo Application's main entry
 *                  point.
 *************************************************************************/

int main ( void )
{
    // Initialize the processor and peripherals.
    if ( InitializeSystem() != TRUE )
    {
        UART2PrintString( "\r\n\r\nCould not initialize USB Custom Demo App - system.  Halting.\r\n\r\n" );
        while (1);
    }
    if ( USBHostInit(0) == TRUE )
    {
        UART2PrintString( "\r\n\r\n***** USB Custom Demo App Initialized *****\r\n\r\n" );
    }
    else
    {
        UART2PrintString( "\r\n\r\nCould not initialize USB Custom Demo App - USB.  Halting.\r\n\r\n" );
        while (1);
    }

    // Main Processing Loop
    while (1)
    {
        // This demo does not check for overcurrent conditions.  See the
        // USB Host - Data Logger for an example of overcurrent detection
        // with the PIC24F and the USB PICtail Plus.

        // Maintain USB Host State
        USBHostTasks();
        // Maintain Demo Application State
        ManageDemoState();
    }

    return 0;

} // main


/*************************************************************************
 * EOF main.c
 */

